from __future__ import annotations

import typing

_DATA_PROXY_SLOTS = ("__data_obj__", "__bool_hook__")

T = typing.TypeVar("T")


class ChainProxy(typing.Generic[T]):
    __slots__ = _DATA_PROXY_SLOTS

    def __init__(self, data_object: T, *, bool_hook: bool = False) -> None:
        self.__data_obj__ = data_object
        self.__bool_hook__ = bool_hook

    def __getattr__(self, attr: str) -> ChainProxy[T]:
        object_attribute = getattr(self.__data_obj__, attr, None)

        if object_attribute is None:
            return ChainProxy(object_attribute, bool_hook=False)

        return ChainProxy(object_attribute, bool_hook=True)

    def __bool__(self) -> bool:
        return self.__bool_hook__

    def __call__(self) -> T:
        return self.__data_obj__

    def __repr__(self) -> str:
        return f"DataProxy(data_object={self.__data_obj__}, bool_hook={self.__bool_hook__})"

    def __str__(self) -> str:
        return f"data_object={self.__data_obj__}, bool_hook={self.__bool_hook__}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['safebag']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'safebag',
    'version': '0.1.2',
    'description': 'Easy Python package for optional chaining pattern',
    'long_description': '# Safebag\n\n_Safebag_ is a little python package implementing optional chaining.\n\n## Installation\n\n```bash\npip install safebag\n```',
    'author': 'Alexandr Solovev',
    'author_email': 'nightingale.alex.info@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/galeNightIn/safebag',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import typing

from ._chain_proxy import ChainProxy


def dataclass_proxy(data_object: typing.Any) -> ChainProxy:
    return ChainProxy(data_object)


def get_value(proxy_object: ChainProxy) -> typing.Any:
    return proxy_object()
